#include "helpers.h"

#include <qapplication.h>

#ifdef QTOPIA
#include <qobjectlist.h>
#include <qpe/config.h>
#else
#include <qptrlist.h>
#include <qobjectlist.h>
#include <config.h>
#endif

QString formatFileSize(long long filesize)
{
	QString result;

	if (filesize > 1024 * 1024 * 1024)
		result = QString().sprintf("%.2f GiB", (float)filesize / (1024 * 1024 * 1024));
	else if (filesize > 1024 * 1024)
		result = QString().sprintf("%.2f MiB", (float)filesize / (1024 * 1024));
	else
		result = QString().sprintf("%.1f KiB", (float)filesize / 1024);

	return result;
}

QString formatBitrate(long long bitrate)
{
	QString result;

	if (bitrate > 1000 * 1000 * 1000)
		result = QString().sprintf("%.2f Gbit/s", (float)bitrate / (1000 * 1000 * 1000));
	else if (bitrate > 1000 * 1000)
		result = QString().sprintf("%.2f Mbit/s", (float)bitrate / (1000 * 1000));
	else if (bitrate > 1000)
		result = QString().sprintf("%.1f kbit/s", (float)bitrate / 1000);
	else
		result = QString().sprintf("%.1f bit/s", (float)bitrate);

	return result;
}

QString formatPlayTime(int lengthInSeconds, bool useShortestFormIfPossible)
{
	uint absLengthInSeconds = (lengthInSeconds < 0) ? -lengthInSeconds : lengthInSeconds;

	uint playtimeMins = absLengthInSeconds / 60;
	uint playtimeSecsRem = absLengthInSeconds % 60;
	uint playtimeHours = playtimeMins / 60;
	uint playtimeMinsRem = playtimeMins % 60;
	uint playtimeDays = playtimeHours / 24;
	uint playtimeHoursRem = playtimeHours % 24;

	QString result;

	if (playtimeDays > 0)
		result = QString().sprintf("%d:%02d:%02d:%02d", playtimeDays, playtimeHoursRem, playtimeMinsRem, playtimeSecsRem);
	else if (playtimeHoursRem > 0)
		result = QString().sprintf("%d:%02d:%02d", playtimeHoursRem, playtimeMinsRem, playtimeSecsRem);
	else if (playtimeMinsRem > 0 || !useShortestFormIfPossible)
		result = QString().sprintf("%d:%02d", playtimeMinsRem, playtimeSecsRem);
	else
		result = QString().sprintf("%d", playtimeSecsRem);

	if (lengthInSeconds < 0)
		result = "-" + result;

	return result;
}

void setStyleRecursively(QWidget *w, QStyle *s)
{
    w->unsetPalette();
    w->setStyle(s);

    const QObjectList *children = w->children();
    if (! children)
        return;

#ifdef QTOPIA
    QObjectListIt childit(*children);
#else
    QPtrListIterator<QObject> childit(*children);
#endif

    QObject *child;
    while ((child = childit.current()) != 0)
    {
        ++childit;

        if (child->isWidgetType() && !child->isA("QPopupMenu"))
        	setStyleRecursively((QWidget *) child, s);
    }
}

void saveWidgetPosition(QWidget *widget, const QString &configName, const QString &groupName)
{
#ifndef QTOPIA
	Config cf(configName, Config::File);
	cf.setGroup(groupName);

	bool wasMaximized = widget->isMaximized();

	cf.writeEntry("isMaximized", wasMaximized);

	if (wasMaximized)
		widget->showNormal();

	cf.writeEntry("x", widget->pos().x());
	cf.writeEntry("y", widget->pos().y());

	cf.writeEntry("width", widget->size().width());
	cf.writeEntry("height", widget->size().height());

	cf.writeEntry(
		"hadFrame",
		widget->pos().x() != widget->geometry().x() || widget->pos().y() != widget->geometry().y()
	);

	if (wasMaximized)
		widget->showMaximized();
#endif
}

void loadWidgetPosition(QWidget *widget, const QString &configName, const QString &groupName, int defX, int defY, int defWidth, int defHeight)
{
#ifndef QTOPIA
	Config cf(configName, Config::File);
	cf.setGroup(groupName);

	QSize size(cf.readNumEntry("width", defWidth), cf.readNumEntry("height", defHeight));
	QPoint point(cf.readNumEntry("x", defX), cf.readNumEntry("y", defY));

	QRect screenRect = QApplication::desktop()->availableGeometry(QApplication::desktop()->primaryScreen());

	if (defX == -1 && point.x() == defX)
		point.setX(QMAX(0, screenRect.x() + (screenRect.width() - size.width()) / 2));

	if (defY == -1 && point.y() == defY)
		point.setY(QMAX(0, screenRect.y() + (screenRect.height() - size.height()) / 2));

	// This is an ugly workaround for when we can't determine the frame offset...
	bool hadFrame = cf.readBoolEntry("hadFrame", true);
	if (!hadFrame)
	{
		int diffX = qApp->mainWidget()->geometry().x() - qApp->mainWidget()->frameGeometry().x();
		int diffY = qApp->mainWidget()->geometry().y() - qApp->mainWidget()->frameGeometry().y();

		point.setX(point.x() - diffX);
		point.setY(point.y() - diffY);
	}

	// Use a two step approach to avoid flickering and resizing and
	// work around an issue with some WMs which causes the windows position
	// to be wrongly reported if we save later on and the user didn't change
	// the position or size of the frame.

	// first resize and positioning so Qt knows what we are up to
	widget->resize(size);
	widget->move(point);
	// show window, so the WM is prepared...
	widget->showNormal();
	// finally set again, so the WM explicitly gets it...
	widget->resize(size);
	widget->move(point);

	if (cf.readBoolEntry("isMaximized", false))
		widget->showMaximized();
#endif
}
